<?php

/***************************************************************************
 *   Copyright (C) 2005 by Ferenc Veres   *
 *   lion@netngine.hu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
include_once('config.inc.php');
include_once('lib/Common.php');

$DB = NewADOConnection(GetConfigVar('db_software'));
$DB->Connect(GetConfigVar('db_host'), GetConfigVar('db_username') , GetConfigVar('db_password'), GetConfigVar('db_database'));
CacheIndexes();

define('LOCATE_CURR', 0);
define('LOCATE_NEXT', 1);
define('LOCATE_NEXTMISSING', 2);
define('LOCATE_PREV', 3);
define('LOCATE_PREVMISSING', 4);

/**
* Base class for translation frontends, implementing the functions frontend events may call.
*
* Programmers can inherit this class in frontend implementations, handle
* the frontend events there, upload user control values to this class' member
* variables and call event handler functions here.
*/

class TranslationBackend
{
	public $Event;	/**< Variable to store the currently processed event's name. */ 

	// CONTROLS
	
	public $txtFindText;	/**< TextBox: the text input for custom sentence search */
	public $ddlSearchType;  /**< DropDownList: Search type, word index, original or translation string parts */
	public $ddlFindPercent;	/**< DropDownList: the sentence search function's match percent limit */
	public $ddlLocaleID;	/**< DropDownList: the ID of the new locale in 'setlocale' event, otherwise overwritten from $xOldLocaleID */
	public $xOldLocaleID;	/**< Previously selected locale ID, kept unless 'setlocale' event occurs. */
	public $xStringID;	/**< One selected string's ID for e.g. string details pages */
	public $xDelStringID;	/**< The string clicked for deleting. */
	public $txtEditString;	/**< String editor when edting a translation */
	
	// OUTPUT

	public $Translations;	/**< Array of TranslationStrings: Search results */
	public $LocatedString;	/**< TranslationString: one string for string details pages */
	public $LocatedProjects;/**< Array of TranslationProjects: e.g. project path for one string */
	public $ChildProjects;	/**< ProjectIterator: child projects for the current location */
	public $CurrentStrings; /**< List of strings in the selected project. */
	public $RootLastSeen;	/**< Timestamp of the root project for dead strings/projects */
	
	// PRIVATE 

	private $String;	/**< TranslationString to do searches */

	/**
	* Create a new TranslationBackend class for GUI operations.
	*
	* Initializes the internal variables and GUI controls with default values.
	* E.g. default match percentage is 60, default locale ID is 1.
	*/
	
	function __construct()
	{
		// Get and use default locale
		$localeName = GetConfigVar('default_locale');
		$localeID = TranslationProject::$LocaleIDCache[$localeName];
		if(empty($localeID))
		{
			throw new Exception("Invalid default locale in the configuration: $localeName");
		}
		
		// Create a new translation string handler class
		$this->String = new TranslationString();
		$this->DB = GetDbConn();
		$this->Tables = GetTables();

		$this->txtFindText = "";
		$this->ddlSearchType = 0;
		$this->ddlFindPercent = 60;
		$this->ddlLocaleID = $localeID;
		$this->xOldLocaleID = $localeID;
		$this->xStringID = 0;
		$this->xProjectID = 0;

		$this->InitLanguage();
	}

	/**
	* Init language change specific variables
	*/
	
	public function InitLanguage()
	{
		// Find a legal Xaraya root project
		$proj = new TranslationProject();
		if($proj->LoadRoot("Xaraya", $this->ddlLocaleID))
		{
			$this->xProjectID = $proj->GetID();
			$this->FindProjectPath();
		}
	}
	
	/**
	* Find a similar strings in the database.
	*
	* Finds the given sentence or similar sentences, where the difference is less than
	* specified in the percentage parameter.
	*
	* INPUT:
	* $txtFindText
	* $ddlLocaleID
	* $ddlFindPercent
	*
	* OUTPUT:
	* $Translations
	*/
	
	function FindStrings()
	{
		switch($this->ddlSearchType)
		{

			case 1:
				// Find in Original as part of the string
				$this->Translations = $this->String->FindText($this->txtFindText, $this->ddlLocaleID, true);
				break;
			
			case 2:
				// Find in Translation as part of the string
				$this->Translations = $this->String->FindText($this->txtFindText, $this->ddlLocaleID, false);
				break;

			default:
				// Similarity matching
				$this->Translations = $this->String->FindSimilar($this->txtFindText, $this->ddlLocaleID, $this->ddlFindPercent);
				break;
		}
	}

	/**
	* Get one string by ID.
	* 
	* The $hdnStringID must be set to the string which you are looking for. Output 
	* will be placed in $LocatedString (a TranslationString object).
	*
	* INPUT:
	* $xStringID
	*
	* OUTPUT:
	* $LocatedString
	*/
	function LocateString($locate = LOCATE_CURR)
	{
		// Load string details
		switch($locate)
		{
			case LOCATE_CURR:
				$this->LocatedString = new TranslationString($this->xStringID);
				break;

			case LOCATE_NEXT:
			case LOCATE_NEXTMISSING:
				$this->LocateStringNext( ($locate == LOCATE_NEXTMISSING) );
				break;
		}
	}

	/**
	* Find the next string after the current one, even across project tree branches.
	*/
	
	private function LocateStringNext($missingOnly)
	{
		unset($this->LocatedString);

		// Skip string-in-project search if xStringID is empty:
		// which means there were no strings in this project

		if($this->xStringID != 0)
		{
			// Take current string and an iterator for it's siblings
			$currentString = new TranslationString($this->xStringID);
			$stringIter = new StringIterator($currentString->GetProjectID());
	
			// FLAT SEARCH - STRINGS OF THE CURRENT PROJECT
			
			// Find current and then take next. If we are last, this will end with false.
			$takeString = false;
			foreach($stringIter as $str)
			{
				if($takeString)
				{
					$this->LocatedString = new TranslationString($str->GetStringID());
	
					// If we find Missing Strings, ignore non-empty results!
					if($missingOnly && $this->LocatedString->GetTranslation() != "")
					{
						// Need to search a missing one, drop that result
						unset($this->LocatedString);
					}
					else
					{
						// Found, break!
						$this->xStringID = $str->GetStringID();
						break;
					}
				}
				
				// If reached the current string, note to take the next one
				if($str->GetStringID() == $currentString->GetStringID())
				{
					$takeString = true;
				}
			}
		}
		
		// RECURSIVE SEARCH - UPWARDS AND DOWNWARDS
		
		// If this was a "last string" in the project, we will search across projects!
		$currentProject = new TranslationProject($this->xProjectID);

		// Stop variable for reaching root
		$rootHit = false;

		// Search next string across projects!
		while(empty($this->LocatedString) && !$rootHit)
		{
			$takeProject = false;

			// Start an iterator on sibling and subprojects
			$parentProjects = new RecursiveIteratorIterator(
				new ProjectIterator($currentProject->GetParentID(), $this->ddlLocaleID),
				RecursiveIteratorIterator::SELF_FIRST);

			// Loop on the projects and find the current one (which hit last string above!)
			foreach($parentProjects as $proj)
			{
				// This happens after the "last-string-hit" project, take next project:
				if($takeProject)
				{
					// Get strings of all next projects and see if there are any strings.
					$stringIter = new StringIterator($proj->GetID());

					// There are strings, take the first and we are done! (in not missing ok)
					if(!$missingOnly)
					{
						// Any strings? If yes, found, break!
						if($stringIter->valid())
						{
							$this->xStringID = $stringIter->current()->GetStringID();
							$this->LocatedString = new TranslationString($this->xStringID);
							break;
						}
					}
					else
					{
						$foundEmpty = false;
						// Missing only, so loop on all strings
						// If we find Missing Strings, ignore non-empty results!
						foreach($stringIter as $str)
						{
							if($str->GetTranslation() == "")
							{
								$this->xStringID = $str->GetStringID();
								$this->LocatedString = new TranslationString($this->xStringID);
								$foundEmpty = true;
								break;
							}
						}

						// If iterator not finished, we found the result
						if($foundEmpty)
						{
							break;
						}
					}
				}

				// Found the "last-string-hit" project?
				if($proj->GetID() == $this->xProjectID)
				{
					$takeProject = true;
				}
			}

			// Stop working if taking parent would result in looping on root!
			$rootHit = ($currentProject->GetParentID() == 0);

			// If not root and not found a string, recurse UPWARDS
			if($currentProject->GetParentID() != 0 && empty($this->LocatedString))
			{
				$currentProject = new TranslationProject($currentProject->GetParentID());
			}

		}
	}
	/**
	* Find the path leading the the selected project (parent projects, child projects).
	*
	* Parent projects are returned in an array, child projects are in a ProjectIterator.
	*
	* INPUT:
	* $xProjectID
	*
	* OUTPUT:
	* $LocatedProjects[]
	* $ChildProjects
	*/
	
	function FindProjectPath()
	{
		unset ($this->LocatedProjects);
		unset ($this->ChildProjects);

		$pid = $this->xProjectID;
		
		// Loop and collect projects until reaching the end
		while($pid != 0)
		{
			$proj = new TranslationProject($pid);
			$this->LocatedProjects[] = $proj;
			$pid = $proj->GetParentID();

			// Store the root project's date for dead strings/projects timestamp
			if($pid == 0)
			{
				$this->RootLastSeen = $proj->GetLastSeen();
			}
		}
		
		// Make a new project Iterator with current parent to see if there are children
		$this->ChildProjects = new ProjectIterator($this->xProjectID, $this->ddlLocaleID);
	}

	/**
	* Save the currently edited string.
	*
	* Updates the edited string in the database.
	*
	* INPUT: $xStringID, $txtEditString
	*/
	 
	function SaveString()
	{
		// Load our string from the DB, change it and save changes
		$string = new TranslationString($this->xStringID);
		$string->SetTranslation($this->txtEditString);
		$string->Save();
	}

	/**
	* Delete one (dead) string from the database.
	*/

	function DeleteString()
	{
		// Load our string
		$string = new TranslationString($this->xDelStringID);
		$string->DestroyData();
	}
	/**
	* Load all strings belonging to a project
	*/
	
	function LoadProjectStrings()
	{
		$this->CurrentStrings = new StringIterator($this->xProjectID);
	}
}

?>
