DROP TABLE IF EXISTS Strings;
DROP TABLE IF EXISTS SentenceIndex;
DROP TABLE IF EXISTS WordIndex;
DROP TABLE IF EXISTS Projects;
DROP TABLE IF EXISTS Locales;

CREATE TABLE Locales (
	ID int NOT NULL PRIMARY KEY auto_increment,
	Name varchar(40) NOT NULL
) CHARACTER SET utf8 COLLATE utf8_bin
  TYPE=INNODB;

CREATE TABLE Projects (
	ID int NOT NULL PRIMARY KEY auto_increment,
	ParentID int NOT NULL,
	Name varchar(40) NOT NULL,
	LocaleID int NOT NULL,
	Skeleton longtext NOT NULL DEFAULT '',
	OrigDoc longtext NOT NULL DEFAULT '',
	LastSeen datetime NULL DEFAULT NULL,
	FOREIGN KEY (LocaleID) REFERENCES Locales (ID)
) CHARACTER SET utf8 COLLATE utf8_bin
  TYPE=INNODB;

CREATE TABLE WordIndex (
	ID int NOT NULL PRIMARY KEY auto_increment,
	Word varchar(255) NOT NULL DEFAULT '',
	UNIQUE INDEX (Word)
) CHARACTER SET utf8 COLLATE utf8_bin
  TYPE=INNODB;

CREATE TABLE SentenceIndex (
	SentenceID int NOT NULL,
	WordID int NOT NULL,
	Occurences int NOT NULL,
	INDEX (SentenceID),
	INDEX (WordID),
	FOREIGN KEY (WordID) REFERENCES WordIndex (ID)
) CHARACTER SET utf8 COLLATE utf8_bin
  TYPE=INNODB;

CREATE TABLE Strings (
	ID int NOT NULL PRIMARY KEY auto_increment,
	ProjectID int NOT NULL,
	Original text NOT NULL,
	SentenceID int NULL DEFAULT NULL,
	LocaleID int NOT NULL,
	Translation text NOT NULL DEFAULT '',
	LastSeen datetime NULL DEFAULT NULL,
	RowOrder int NOT NULL DEFAULT 0,
	FOREIGN KEY (ProjectID) REFERENCES Projects (ID),
	FOREIGN KEY (SentenceID) REFERENCES SentenceIndex (SentenceID),
	FOREIGN KEY (LocaleID) REFERENCES Locales (ID)
) CHARACTER SET utf8 COLLATE utf8_bin
  TYPE=INNODB;

INSERT INTO Locales (Name) VALUES ("hu_HU.utf-8");
INSERT INTO Locales (Name) VALUES ("pt_BR.utf-8");
INSERT INTO Locales (Name) VALUES ("ru_RU.utf-8");
INSERT INTO Locales (Name) VALUES ("da_DK.utf-8");
INSERT INTO Locales (Name) VALUES ("sv_SE.utf-8");
INSERT INTO Locales (Name) VALUES ("nl_NL.utf-8");
INSERT INTO Locales (Name) VALUES ("ph_PH.utf-8");

