<?
/***************************************************************************
 *   Copyright (C) 2005 by Ferenc Veres   *
 *   lion@netngine.hu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 include_once('TranslationString.php');

/**
* Class to list and iterate TranslationString objects as they are stored in the database.
*
* StringIterator can be initialized with a project ID and then strings belonging to
* that project can be looped.
*/

class StringIterator implements Iterator
{
	private $ProjectID;		/**< Stores which project's strings are loaded. */
	private $ProjectStrings;	/**< Loaded strings. */

	private $Current;		/**< Index of the current string in the iterator */
	private $IsValid;		/**< Iterator valid or not. */

	/**
	* Create a new TranslationString iterator for looping on contained strings.
	*
	* @param $projectID Which project's strings to load.
	*/
	public function __construct($projectID)
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$this->ProjectID = $projectID;

		// Fetch all strings of the project from the DB.
		$rs = $DB->Execute("SELECT ID, Original, LocaleID, Translation, SentenceID, LastSeen, RowOrder FROM ".$Tables["Strings"]." WHERE ProjectID=? ORDER BY RowOrder", array($this->ProjectID));

		while($row = $rs->FetchRow())
		{
			// Add fully initializes strings to the array.
			$this->ProjectStrings[] = new TranslationString(
				$row["ID"],
				$this->ProjectID,
				$row["Original"],
				$row["LocaleID"],
				$row["Translation"],
				$row["SentenceID"],
				$row["LastSeen"],
				$row["RowOrder"]);
		}
		$this->rewind();
	}

	/* Iterator interface */

	/**
	* Iterator interface: rewind function.
	*/
	public function rewind()
	{
		$this->Current = 0;
	}
	
	/**
	* Iterator interface: hasMore function.
	*/
	public function hasMore()
	{
		return (count($this->ProjectStrings) > $this->Current);
	}
	
	/**
	* Iterator interface: key function.
	*/
	public function key()
	{
		return $this->Current;
	}
	
	/**
	* Iterator interface: current function.
	*/
	public function current()
	{
		return $this->ProjectStrings[$this->Current];
	}
	
	/**
	* Iterator interface: next function.
	*/
	public function next()
	{
		$this->Current++;
	}
	
	/**
	* Iterator interface: valid function.
	*/
	public function valid()
	{
		return (count($this->ProjectStrings) > $this->Current);
	}
}